import '../lib/key_value_store_desktop.dart';

void main(List<String> arguments) async {
  final kvs = await DesktopKeyValueStore.getInstance();

  print('Running key value store tests for platform [${arguments.first}]...');
  await _testGetKeys(kvs);
  await _testBools(kvs);
  await _testInts(kvs);
  await _testDoubles(kvs);
  await _testStrings(kvs);
  await _testStringLists(kvs);
  await _testRemove(kvs);
  await _testClear(kvs);
  print('Tests passed.');
}

Future<void> _testGetKeys(DesktopKeyValueStore kvs) async {
  await kvs.clear();

  await kvs.setString('myKey1', 'myValue1');
  await kvs.setString('myKey2', 'myValue2');
  await kvs.setString('myKey3', 'myValue3');

  final keys = kvs.getKeys().toList();
  assert(keys.length == 3);
  assert(keys[0] == 'myKey1');
  assert(keys[1] == 'myKey2');
  assert(keys[2] == 'myKey3');
}

Future<void> _testBools(DesktopKeyValueStore kvs) async {
  await kvs.clear();

  await kvs.setBool('myTrueBool', true);
  await kvs.setBool('myFalseBool', false);

  assert(kvs.getBool('myTrueBool') == true);
  assert(kvs.getBool('myFalseBool') == false);
  assert(kvs.getBool('myNullBool') == null);
}

Future<void> _testInts(DesktopKeyValueStore kvs) async {
  await kvs.clear();

  await kvs.setInt('myInt', 1337);
  assert(kvs.getInt('myInt') == 1337);
}

Future<void> _testDoubles(DesktopKeyValueStore kvs) async {
  await kvs.clear();

  await kvs.setDouble('myDouble', 1337.7331);
  assert(kvs.getDouble('myDouble') == 1337.7331);
}

Future<void> _testStrings(DesktopKeyValueStore kvs) async {
  await kvs.clear();

  await kvs.setString('myString', 'Hello World!');
  assert(kvs.getString('myString') == 'Hello World!');
}

Future<void> _testStringLists(DesktopKeyValueStore kvs) async {
  await kvs.clear();

  await kvs.setStringList('myStringList', ['value #1', 'value #2']);

  final list = kvs.getStringList('myStringList');
  assert(list.length == 2);
  assert(list[0] == 'value #1');
  assert(list[1] == 'value #2');
}

Future<void> _testRemove(DesktopKeyValueStore kvs) async {
  await kvs.clear();

  await kvs.setString('myToBeRemovedString', 'Hello World!');
  assert(kvs.getString('myToBeRemovedString') == 'Hello World!');

  await kvs.remove('myToBeRemovedString');
  assert(kvs.getString('myToBeRemovedString') == null);
}

Future<void> _testClear(DesktopKeyValueStore kvs) async {
  await kvs.clear();

  await kvs.setString('myToBeRemovedString1', 'Hello World from me!');
  await kvs.setString('myToBeRemovedString2', 'Hello World from me too!');
  await kvs.setString('myToBeRemovedString3', 'Hello World from me three!');

  assert(kvs.getString('myToBeRemovedString1') == 'Hello World from me!');
  assert(kvs.getString('myToBeRemovedString2') == 'Hello World from me too!');
  assert(kvs.getString('myToBeRemovedString3') == 'Hello World from me three!');

  await kvs.clear();

  assert(kvs.getString('myToBeRemovedString1') == null);
  assert(kvs.getString('myToBeRemovedString2') == null);
  assert(kvs.getString('myToBeRemovedString3') == null);
}
