import 'dart:convert';

import 'package:key_value_store/key_value_store.dart';
import 'package:leveldb/leveldb.dart';

class DesktopKeyValueStore implements KeyValueStore {
  static const _dbName = 'flutter_desktop_key_value_store.db';

  DesktopKeyValueStore._(this._db);
  final LevelDB _db;

  static DesktopKeyValueStore _instance;
  static Future<DesktopKeyValueStore> getInstance() async {
    if (_instance == null) {
      final db = await LevelDB.openUtf8(_dbName);
      _instance = DesktopKeyValueStore._(db);
    }
    return _instance;
  }

  @override
  Future<bool> clear() async {
    for (var key in getKeys()) {
      await remove(key);
    }

    return true;
  }

  @override
  bool getBool(String key) {
    if (_db.get(key) == 'true') {
      return true;
    }
    if (_db.get(key) == 'false') {
      return false;
    }

    // TODO (roughike): Add a fallback option for booleans.
    return null;
  }

  @override
  double getDouble(String key) => double.parse(_db.get(key));

  @override
  int getInt(String key) => int.parse(_db.get(key));

  @override
  Set<String> getKeys() => _db.getItems().keys.toSet();

  @override
  String getString(String key) => _db.get(key);

  @override
  List<String> getStringList(String key) =>
      (json.decode(_db.get(key)) as List).cast<String>();

  @override
  Future<bool> remove(String key) async {
    _db.delete(key);
    return true;
  }

  @override
  Future<bool> setBool(String key, bool value) => _setValue(key, value);

  @override
  Future<bool> setDouble(String key, double value) => _setValue(key, value);

  @override
  Future<bool> setInt(String key, int value) => _setValue(key, value);

  @override
  Future<bool> setString(String key, String value) => _setValue(key, value);

  @override
  Future<bool> setStringList(String key, List<String> values) =>
      _setValue(key, values);

  Future<bool> _setValue(String key, dynamic value) {
    if (value is String) {
      _db.put(key, value);
    } else if (value is bool || value is double || value is int) {
      _db.put(key, value.toString());
    } else if (value is List) {
      _db.put(key, json.encode(value));
    }

    return Future.value(true);
  }
}
